local addon, CF = ...
LibStub("AceAddon-3.0"):NewAddon(CF, addon,"AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addon)
local symbols = "!@$%^&*"
local numSymbols = #symbols

local defaults = {
	profile = {
		events = {
			["CHANNEL"] = true,
			["SAY"] = true,
			["YELL"] = true,
			["EMOTE"] = true,
			
			["WHISPER"] = true,
			["WHISPER_INFORM"] = true,
			["BN_WHISPER"] = true,
			
			["GUILD"] = true,
			["OFFICER"] = true,
			
			["BATTLEGROUND"] = true,
			["BATTLEGROUND_LEADER"] = true,
			["PARTY"] = true,
			["PARTY_LEADER"] = true,
			["RAID"] = true,
			["RAID_LEADER"] = true,
			["RAID_WARNING"] = true,
		},
		filtered = {},
		whitelist = {},
		blacklist = {},
		wholeWordOnly = true,
		removeLines = false,
	}
}

function CF:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New(addon .."DB", defaults, true)
	self:RegisterOptions()
	self:RegisterChatCommand(L["customfilter"], "HandleChatCommand")
	self:RegisterChatCommand(L["cf"], "HandleChatCommand")
end

function CF:OnEnable()
	for event, enabled in pairs(self.db.profile.events) do
		if enabled then
			ChatFrame_AddMessageEventFilter("CHAT_MSG_".. event, self.FilterFunc)
		end
	end
end

function CF:OnDisable()
	for event, _ in pairs(self.db.profile.events) do
		ChatFrame_RemoveMessageEventFilter(event, self.FilterFunc)
	end
end

function CF:MaskString(num)
	local rep = self.db.profile.replacement
	if rep then
		return rep
	else
		local str, ind;
		for i = 1, num do
			ind = random(numSymbols)
			str = (str or "") .. symbols:sub(ind, ind)
		end
		return str
	end
end

local function filter(msg)
	local count = 0
	local self = CF
	local new = msg:gsub("(%S+)", function(word)
		local start, finish;
		if self.db.profile.wholeWordOnly then
			if self.db.profile.filtered[word:lower()] then
				word = self:MaskString(#word)
				count = count + 1
			end
		else
			for filterWord, _ in pairs(self.db.profile.filtered) do
				start, finish = word:match("^(%S*)".. filterWord .."(%S*)$")
				if start or finish then
					word = strconcat(start or "", self:MaskString(#filterWord), finish or "")
					count = count + 1
				end
			end
		end
		return word
	end)
	return new, count
end

function CF.FilterFunc(chatFrame, event, msg, author, ...)
	local self = CF
	local lower = author:lower()
	if not self.db.profile.whitelist[lower] then --Don't filter messages from people on the whitelist
		local newMsg, numReplacements = filter(msg)
		if self.db.profile.blacklist[lower] or numReplacements > 0 then
			return self.db.profile.removeLines, newMsg, author, ...
		end
	end
end