local addon, CF = ...
local ACD = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addon)
local frameref;
local path = {}

local function dump(...)
	CHOONDUMP = {...}
	SlashCmdList.DUMP("CHOONDUMP")
end

local eventLocales = {
	["CHANNEL"] = false,
	["SAY"] = false,
	["YELL"] = false,
	["EMOTE"] = false,
	
	["WHISPER"] = false,
	["WHISPER_INFORM"] = L["Outgoing Whisper"],
	["BN_WHISPER"] = false,
	
	["GUILD"] = false,
	["OFFICER"] = false,
	
	["BATTLEGROUND"] = false,
	["BATTLEGROUND_LEADER"] = false,
	["PARTY"] = false,
	["PARTY_LEADER"] = false,
	["RAID"] = false,
	["RAID_LEADER"] = false,
	["RAID_WARNING"] = false,
	
}

local function InputSet(info, input)
	local opt = info[#info - 1]
	input = input:lower():trim()
	input = input:gsub("[%[%]%(%)%%%{%}%.%,%:%;]", "")
	if input == "" then return end
	if CF.db.profile[opt][input] then
		CF.db.profile[opt][input] = nil
	else
		CF.db.profile[opt][input] = true
	end
	
	-- for i = 1, #info-1 do
		-- path[i] = info[i]
	-- end
	-- tinsert(path, "display")
	-- dump(info.options)
	-- ACD:Open(addon, info.options, unpack(path)) --Refresh the display list
	-- wipe(path)
end

local function DisplayGet(info)
	local opt = info[#info - 1]
	local str = next(CF.db.profile[opt])
	for word, _ in next, CF.db.profile[opt], str do
		str = str .. "\n" .. word
	end
	
	return str
end

local options = {
	type = "group",
	args = {
		enable = {
			name = L["Enable"],
			desc = L["Enables / disables the addon"],
			type = "toggle",
			set = function(info, val) if val then CF:Enable() CF:Print(L["Enabled"]) else CF:Disable() CF:Print(L["Disabled"])end end,
			get = function(info) return CF:IsEnabled() end
		},
		events = {
			type = "group",
			name = L["Chat Types"],
			args = {
				description = {
					type = "description",
					name = L["Select the chat types that will be filtered."]
				},
				select = {
					name = "",
					type = "multiselect",
					values = function()
						local t = {}
						for event, name in pairs(eventLocales) do
							t[event] = name or _G[event]
						end
						return t
					end,
					get = function(info, event) return CF.db.profile.events[event] end,
					set = function(info, event, val)
						CF.db.profile.events[event] = val
						_G[("ChatFrame_%sMessageEventFilter"):format(val and "Add" or "Remove")]("CHAT_MSG_".. event, CF.FilterFunc)
					end,
				},
			},
		},
		filtered = {
			name = L["Filtered Words"],
			type = "group",
			args = {
				description = {
					type = "description",
					name = L["Change which words are filtered."]
				},
				wholeWordOnly = { --The aa_ part just puts this before the other parts when the options are created
					type = "toggle",
					name = L["Whole Word Only"],
					desc = L["WHOLE_WORD_DESC"],
					get = function(info) return CF.db.profile.wholeWordOnly end,
					set = function(info, val) CF.db.profile.wholeWordOnly = val end,
				},
				removeLines = {
					type = "toggle",
					name = L["Remove chat lines"],
					desc = L["REMOVE_LINES_DEC"],
					get = function(info) return CF.db.profile.removeLines end,
					set = function(info, val) CF.db.profile.removeLines = val end,
				},
				input = {
					type = "input",
					name = L["Add/Remove"],
					desc = L["Add or remove a word from the filter list"],
					set = InputSet,
					width = "full"
				},
				display = {
					type = "input",
					name = L["Filter List"],
					desc = L["These words will be replaced with the specified replacement word or a series of random symbols in your chat."],
					get = DisplayGet,
					-- disabled = true,
					multiline = 20,
					width = "full",
					dialogControl = "CustomFilter_DisplayBox"
				},
				replacement = {
					type = "input",
					name = L["Replacement word"],
					desc = L["Filtered words will be replaced with this. Spaces are allowed. If empty, random symbols will be used instead of a fixed word."],
					get = function(info) return CF.db.profile.replacement or "" end,
					set = function(info, val) CF.db.profile.replacement = (val == "" and nil or val) end,
				},
			},
		},
		whitelist = {
			type = "group",
			name = L["Whitelist"],
			args = {
				description = {
					type = "description",
					name = L["Prevent messages received from certain players from being filtered."]
				},
				input = {
					type = "input",
					name = L["Add/Remove"],
					desc = L["Add or remove a player from the whitelist"],
					set = InputSet,
					width = "full"
				},
				display = {
					type = "input",
					name = L["Whitelist"],
					desc = L["Messages from these players won't be filtered."],
					get = DisplayGet,
					multiline = 20,
					-- disabled = true,
					width = "full",
					dialogControl = "CustomFilter_DisplayBox"
				},
			},
		},
		blacklist = {
			type = "group",
			name = L["Blacklist"],
			args = {
				description = {
					type = "description",
					name = L["Always filter messages received from certain players."]
				},
				input = {
					type = "input",
					name = L["Add/Remove"],
					desc = L["Add or remove a player from the blacklist"],
					set = InputSet,
					width = "full"
				},
				display = {
					type = "input",
					name = L["Blacklist"],
					desc = L["Messages from these players will always be filtered. Works best with \"Remove chat ines\" enabled."],
					get = DisplayGet,
					multiline = 20,
					-- disabled = true,
					width = "full",
					dialogControl = "CustomFilter_DisplayBox"
				},
			},
		},
	},
}

function CF:RegisterOptions()
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(addon, options)
	frameref = ACD:AddToBlizOptions(addon)
end

function CF:HandleChatCommand()
	InterfaceOptionsFrame_OpenToCategory(frameref)
end